@@ -203,12 +203,13 @@ var MediaSniffer = {
           contentType = this.mediaTypesRx.test(channel.contentType)
             ? channel.contentType
             : Youtube.get_stream_content_type(channel.URI.spec);
-          if (Object.keys(map).some(function(url){return Youtube.stream_url_equals(url, channel.URI.spec);})) {
+          if (Object.keys(map).some(function(url){return !map[url].parsed && Youtube.stream_url_equals(url, channel.URI.spec);})) {
             if ( ! extras["flashgot::parsed"] && ! /^audio\//.test(contentType)) {
               this._set_current_url(win, channel.URI.spec);
             }
             return;
           }
+          extras["flashgot::Youtube"] = Youtube; // for flashgotOverlay.js
         }
         if (!contentType) {
           contentType = channel.contentType;
@@ -248,6 +249,7 @@ var MediaSniffer = {
             url = this.normalizeQuery(url.replace(/&range=[^&]*|\/frag\(\d+\)/g, '').replace(/\b(?:cm2|mt)=\d+\b/, ''));
             const goodDMHost = "vid2.ec.dmcdn.net"; // Dailymotion unfragmented-enabled host
             if (url in map || url.replace(/(:\/\/)[^\/]+\.(?:dmcdn\.net|dailymotion\.com)(?=\/)/, "$1" + goodDMHost) in map) return;
+            if ( ! extras.window) { extras.window = yt_win; }
             if (this.peek(url, extras, function (xhr) {
               switch (xhr.status) {
               case 403:
@@ -332,7 +334,8 @@ var MediaSniffer = {
               : (Math.round(contentLength / 1048576)) + "MB";
 
             // Youtube channel hack
-            var doc = yt_win.document;
+            var title_win = fg.getPref("media.use_iframe_title", false) ? yt_win : win;
+            var doc = title_win.document;
             var node = doc.getElementById("playnav-curvideo-title");
 
             var createStrings = function (title, o) {
@@ -349,12 +352,12 @@ var MediaSniffer = {
               var fname = typeInfo.fname || '';
 
               title = title.replace(/^\s+|\s+$/, '');
-              const unicode = fg.getPref("media.unicode") && /^UTF.?8$/i.test(yt_win.document.characterSet);
+              const unicode = fg.getPref("media.unicode") && /^UTF.?8$/i.test(title_win.document.characterSet);
               if (title) {
                 const nonWordRx = unicode ? /[\u0000-\u0020\u2000-\u206F\u2E00-\u2E7F]+|[^\w\u0080-\uffff]+/g : /\W+/g;
                 // remove site name from title
                 title = title.replace(new RegExp("\\b(?:" +
-                  (yt_win.location.host || '').split(".").filter(function (s) {return s}).join("|") + ")\\b", 'ig'), '')
+                  (title_win.location.host || '').split(".").filter(function (s) {return s}).join("|") + ")\\b", 'ig'), '')
                   .replace(/https?:\/{2}/gi, '')
                   .replace(nonWordRx, '_')
                   .replace(/^_+([^_])/g, '$1')
@@ -398,8 +401,11 @@ var MediaSniffer = {
                 parsed: extras["flashgot::parsed"],
                 seek_pos: extras["flashgot::seek_pos"],
                 sort_key: extras["flashgot::sort_key"],
-                Youtube: extras["flashgot::parsed"] ? Youtube : null,
-                yt_dash: extras["flashgot::yt_dash"]
+                Youtube: extras["flashgot::Youtube"],
+                yt_dash: extras["flashgot::yt_dash"],
+                video_id: extras["flashgot::video_id"],
+                title: extras["flashgot::parsed"] ? extras.title.replace(/^\s+|\s+$/g, "") || ("Video " + extras["flashgot::video_id"]): "",
+                video_link: extras["flashgot::video_link"]
               }));
               fg.interceptor.extractPostData(channel, entry); // adds entry.postData if needed
               // YouTube fallback host.
