@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -144,16 +144,20 @@ RedirectContext.prototype = {
     }
   },
   
-  createReq: function() {
-    return CC["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(CI.nsIXMLHttpRequest);
+  createReq: function(method, url) {
+    var xhr = CC["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(CI.nsIXMLHttpRequest);
+    xhr.open(method, url);
+    xhr.channel.loadFlags |= xhr.channel.INHIBIT_PERSISTENT_CACHING;
+    if (fg.inPrivate) fg.privatize(xhr.channel);
+    return xhr;
   },
   
   load: function(url, callbacks, data) {
     
     if (typeof(data) == "undefined") data = null;
     
-    var req = this.createReq();
-    req.open(data == null ?  "GET" :"POST", url, true);
+    var req = this.createReq(data == null ?  "GET" :"POST", url);
+   
     if (data != null) req.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
     
     if (typeof(callbacks) != "object") {
@@ -186,7 +190,7 @@ RedirectContext.prototype = {
     
     this.start();
     try {
-      req.send(data); 
+      req.send(data);
     } catch(e) {
       this.done();
     }
@@ -254,9 +258,9 @@ RedirectContext.prototype = {
     },
     
     function depositfiles_com(l, context) {
-      if (/^http:\/\/depositfiles\.com\//.test(l.href))
+      if (/^http:\/\/(?:depositfiles\.(?:org|com)|dfiles\.eu)\//.test(l.href))
         context.load(l.href, function(req) {
-          var m = req.responseText.match(/https?:\/\/[^"'\s]+depositfiles\.com\/auth\-[^"' ]+/);
+          var m = req.responseText.match(/https?:\/\/[^"'\s]+(?:depositfiles\.(?:org|com)|dfiles\.eu)\/auth\-[^"' ]+/);
           if (m) context.change(l, m[0]);
         });
     },
@@ -302,18 +306,18 @@ RedirectContext.prototype = {
     function imagebam_com(l, context) {
       if (/^http:\/\/[\\w\.]*?imagebam\.com\/image\/\w+/.test(l.href)) {
         context.load(l.href, function(req) {
-          var m = req.responseText.match(/http:\/\/[0-9]+\.imagebam\.com\/dl\.php\?ID=[^"']*/i);
+          var m = req.responseText.match(/\bhttps?:\/\/[0-9]+\.imagebam\.com\/d(?:dl\.php\?ID=[^"']+|ownload\/[^"']+\?download=[^"']*)/i);
           if (m) context.change(l, m[0]);
         });
       }
     },
     
     function imagefap_com(l, context) {
-      if (!/^https?:\/\/(?:[^\.\/]+\.)?imagefap\.com\/(?:image\.php|(?:gallery|pictures)[\.\/])/.test(l.href))
+      if (!/^https?:\/\/(?:[^\.\/]+\.)?imagefap\.com\/(?:gallery|pictures|photo)[\.\/]/.test(l.href))
         return;
 
       var processedBy = arguments.callee.name;
-      if (/imagefap\.com\/image\.php\?id=/.test(l.href)) {
+      if (/imagefap\.com\/photo\//.test(l.href)) {
         context.load(l.href, function(req) {
           var t = req.responseText;
           var url;
@@ -351,7 +355,7 @@ RedirectContext.prototype = {
          
           var t = req.responseText.replace(/relatedgalleriescontainer[\s\S]+function\s+showMoreGalleries/, '');
          
-          var m = t.match(/<a[^>]*"\/image\.php?[^"]*/ig);
+          var m = t.match(/<a[^>]*"\/(?:image\.php|photo\/)?[^"]*/ig);
           if (!m) return;
           
           for (var j = 0; j < m.length; j++) {
@@ -605,8 +609,7 @@ RedirectContext.prototype = {
           return;
         }
         
-        var xhr = context.createReq();
-        xhr.open("GET", l.href);
+        var xhr = context.createReq("GET", l.href);
         if (xhr.channel instanceof CI.nsIHttpChannel) xhr.channel.redirectionLimit = 0;
         xhr.addEventListener("readystatechange", function() {
           if (xhr.readyState != 4) return;
@@ -1104,16 +1107,21 @@ RedirectContext.prototype = {
     function generic(l, context) {
       if (l.contentType || l.noRedir) return; // avoid jamming FlashGot Media or already redirected URLs
       
-      if (typeof(context.genericRx) != "object") {
+      if (typeof(context.genericRx) !== "object") {
+        try {
+          var exceptions = context.dm.service.getPref("redir.generic.exceptions", "");
+          if (exceptions) context.genericExceptionsRx = new RegExp(exceptions.split(/\s+/).join("|"), "i");
+        } catch (e) {}
         try {
           context.genericRx = new RegExp(context.dm.service.getPref("redir.generic.rx", null), "i");
         } catch(e) {
           context.genericRx = null;
         }
       }  
-      if (context.genericRx == null) return;
-
-      var m = l.href.match(context.genericRx);
+      if (context.genericRx === null) return;
+      var url = l.href.replace(/\b(?:ref|orig|parent|from|s(?:ou)?rc)\w*=[^&]+/g, '');
+      var m = !(context.genericExceptionsRx && context.genericExceptionsRx.test(url)) &&
+              url.match(context.genericRx);
       if (m) {
         var href = m[1];
         context.change(l, (/^https?%3a/i.test(href) ? unescape(href) : href).replace(/^(https?):\/+/i, '$1://'), null, true); // latest arg -> add, rather than replace
