@@ -53,7 +53,7 @@ var zip_slide;
 var zip_wp;			// current position in slide
 var zip_fixed_tl = null;	// inflate static
 var zip_fixed_td;		// inflate static
-var zip_fixed_bl, fixed_bd;	// inflate static
+var zip_fixed_bl, zip_fixed_bd;	// inflate static
 var zip_bit_buf;		// bit buffer
 var zip_bit_len;		// bits in bit buffer
 var zip_method;
@@ -805,8 +805,7 @@ function swf_inflate(data)
 	if (sig === "FWS") { return data; }
 	if (sig !== "CWS") { throw new Error("Invalid signature: " + sig); }
 	var ss = new SwfStream(data);
-	ss.skip_bytes(3); // Signature.
-	ss.read_uint8(); // Version.
+	ss.skip_bytes(4); // Signature, version.
 	var inflated_size = ss.read_uint32();
 	var inflated = zip_inflate(data, 8);
 	if (inflated.length + 8 !== inflated_size) { throw new Error("Inflated size mismatch: expected " + inflated_size + ", got " + inflated.length); }
@@ -822,12 +821,12 @@ function SwfStream(data)
 	var m_buf = 0;
 	var m_buf_bits = 0;
 
-	var byte_align = this.byte_align = function $SwfStream_$byte_align()
+	var byte_align = this.byte_align = function()
 	{
 		m_buf_bits = 0;
 	};
 
-	var read_bits = this.read_bits = function $SwfStream_$read_bits(count)
+	var read_bits = this.read_bits = function(count)
 	{
 		var rc = 0;
 		while (count !== 0)
@@ -851,14 +850,14 @@ function SwfStream(data)
 	};
 
 
-	var read_uint8 = this.read_uint8 = function $SwfStream_$read_uint8()
+	var read_uint8 = this.read_uint8 = function()
 	{
 		byte_align();
 		return read_bits(8);
 	};
 
 
-	var read_uint16 = this.read_uint16 = function $SwfStream_$read_uint16()
+	var read_uint16 = this.read_uint16 = function()
 	{
 		var b1 = read_uint8();
 		var b2 = read_uint8() << 8;
@@ -866,37 +865,37 @@ function SwfStream(data)
 	};
 
 
-	var read_uint32 = this.read_uint32 = function $SwfStream_$read_uint32()
+	var read_uint32 = this.read_uint32 = function()
+	{
+		return read_sint32() >>> 0;
+	};
+
+
+	var read_sint32 = this.read_sint32 = function()
 	{
 		var b1 = read_uint8();
 		var b2 = read_uint8() << 8;
 		var b3 = read_uint8() << 16;
 		var b4 = read_uint8() << 24;
-		return (b1 | b2 | b3 | b4) >>> 0;
+		return b1 | b2 | b3 | b4;
 	};
 
 
-	var read_sint32 = this.read_sint32 = function $SwfStream_$read_sint32()
-	{
-		return read_uint32() | 0;
-	};
-
-
-	var read_fixed8 = this.read_fixed8 = function $SwfStream_$read_fixed8()
+	var read_fixed8 = this.read_fixed8 = function()
 	{
 		var a = read_uint8();
 		var b = read_uint8();
 		return b + (a / 0x100);
 	};
 
-	var read_fixed = this.read_fixed = function $SwfStream_$read_fixed()
+	var read_fixed = this.read_fixed = function()
 	{
 		var a = read_uint16();
 		var b = read_uint16();
 		return b + (a / 0x10000);
 	};
 
-	var read_bytes = this.read_bytes = function $SwfStream_$read_bytes(count)
+	var read_bytes = this.read_bytes = function(count)
 	{
 		byte_align();
 		if (m_byte_idx + count > m_bytes.length)
@@ -908,25 +907,25 @@ function SwfStream(data)
 	};
 
 
-	var skip_bytes = this.skip_bytes = function $SwfStream_$skip_bytes(count)
+	var skip_bytes = this.skip_bytes = function(count)
 	{
 		byte_align();
 		m_byte_idx += count;
 	};
 
-	var n_hbits8 = this.n_hbits8 = function $SwfStream_$n_hbits8(n, count)
+	var n_hbits8 = this.n_hbits8 = function(n, count)
 	{
 		return (n & 0xff) >> (8 - count);
 	};
 
 
-	var n_hbits16 = this.n_hbits16 = function $SwfStream_$n_hbits16(n, count)
+	var n_hbits16 = this.n_hbits16 = function(n, count)
 	{
 		return (n & 0xffff) >> (16 - count);
 	};
 
 
-	var n_lbits16 = this.n_lbits16 = function $SwfStream_$n_lbits16(n, count)
+	var n_lbits16 = this.n_lbits16 = function(n, count)
 	{
 		return n & ((1 << count) - 1);
 	};
@@ -1000,13 +999,13 @@ function doABCReader(bytes)
 	var m_bytes = bytes;
 	var m_byte_idx = 0;
 
-	var eof = this.eof = function $doABCReader_$eof()
+	var eof = this.eof = function()
 	{
 		return m_byte_idx >= m_bytes.length;
 	};
 
 
-	var read_bytes = this.read_bytes = function $doABCReader_$read_bytes(count)
+	var read_bytes = this.read_bytes = function(count)
 	{
 		var end = m_byte_idx + count;
 		if (end > m_bytes.length)
@@ -1019,7 +1018,7 @@ function doABCReader(bytes)
 		return rc;
 	};
 
-	var read_u8 = this.read_u8 = function $doABCReader_$read_u8()
+	var read_u8 = this.read_u8 = function()
 	{
 		if (eof())
 		{
@@ -1029,7 +1028,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_u16 = this.read_u16 = function $doABCReader_$read_u16()
+	var read_u16 = this.read_u16 = function()
 	{
 		var b1 = read_u8();
 		var b2 = read_u8();
@@ -1037,7 +1036,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_s24 = this.read_s24 = function $doABCReader_$read_s24()
+	var read_s24 = this.read_s24 = function()
 	{
 		var b1 = read_u8();
 		var b2 = read_u8() << 8;
@@ -1052,7 +1051,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_u32 = this.read_u32 = function $doABCReader_$read_u32()
+	var read_u32 = this.read_u32 = function()
 	{
 		var n = 0;
 		for (var i = 0; i !== 5; ++i)
@@ -1085,28 +1084,29 @@ function doABCReader(bytes)
 	};
 
 
-	var read_u30 = this.read_u30 = function $doABCReader_$read_u30()
+	var read_u30 = this.read_u30 = function()
 	{
 		// 0x3fffffff is 2^30 - 1.
 		return read_u32() & 0x3fffffff;
 	};
 
 
-	var read_s32 = this.read_s32 = function $doABCReader_$read_s32()
+	var read_s32 = this.read_s32 = function()
 	{
 		return read_u32() | 0;
 	};
 
-	var read_d64 = this.read_d64 = function $doABCReader_$read_d64()
+	var read_d64 = this.read_d64 = function()
 	{
 		// static const boolean little_endian = new Uint8Array(new Uint16Array([1]).buffer)[0] == 1;
-		if (arguments.callee["doABCReader::little_endian"] === undefined)
+		var _static = arguments.callee;
+		if (_static["doABCReader::little_endian"] === undefined)
 		{
-			arguments.callee["doABCReader::little_endian"] = new Uint8Array(new Uint16Array([1]).buffer)[0] === 1;
+			_static["doABCReader::little_endian"] = new Uint8Array(new Uint16Array([1]).buffer)[0] === 1;
 		}
 		var b = new Array(8);
 		for (var i = 0; i !== 8; ++i) { b[i] = read_u8(); }
-		if (!arguments.callee["doABCReader::little_endian"])
+		if ( ! _static["doABCReader::little_endian"])
 		{
 			b.reverse();
 		}
@@ -1114,10 +1114,10 @@ function doABCReader(bytes)
 	};
 
 
-	var read_cpool_info = this.read_cpool_info = function $doABCReader_$read_cpool_info()
+	var read_cpool_info = this.read_cpool_info = function()
 	{
 		var cp = new doabc_cpool_info();
-		
+
 		var cnt;
 
 		cnt = read_u30();
@@ -1201,20 +1201,14 @@ function doABCReader(bytes)
 	};
 
 
-	var read_string_info = this.read_string_info = function $doABCReader_$read_string_info()
+	var read_string_info = this.read_string_info = function()
 	{
 		var size = read_u30();
-		if (size === 0) { return ""; }
-		var arr_utf8 = new Array(size);
-		for (var i = 0; i !== size; ++i)
-		{
-			arr_utf8[i] = String.fromCharCode(read_u8());
-		}
-		return UTF8.decode(arr_utf8.join(""));
+		return size !== 0 ? utf8_decode(read_bytes(size)) : "";
 	};
 
 
-	var read_namespace_info = this.read_namespace_info = function $doABCReader_$read_namespace_info()
+	var read_namespace_info = this.read_namespace_info = function()
 	{
 		var o = {}; //new doabc_namespace_info();
 		o.kind = read_u8();
@@ -1223,7 +1217,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_ns_set_info = this.read_ns_set_info = function $doABCReader_$read_ns_set_info()
+	var read_ns_set_info = this.read_ns_set_info = function()
 	{
 		var o = {}; //new doabc_ns_set_info();
 		var cnt = read_u8();
@@ -1236,7 +1230,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_multiname_info = this.read_multiname_info = function $doABCReader_$read_multiname_info()
+	var read_multiname_info = this.read_multiname_info = function()
 	{
 		var o;
 		var kind = read_u8();
@@ -1270,7 +1264,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_method_info = this.read_method_info = function $doABCReader_$read_method_info()
+	var read_method_info = this.read_method_info = function()
 	{
 		var o = {}; //new doabc_method_info();
 
@@ -1310,7 +1304,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_metadata_info = this.read_metadata_info = function $doABCReader_$read_metadata_info()
+	var read_metadata_info = this.read_metadata_info = function()
 	{
 		var o = {}; //new doabc_metadata_info();
 		o.name = read_u30();
@@ -1324,7 +1318,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_instance_info = this.read_instance_info = function $doABCReader_$read_instance_info()
+	var read_instance_info = this.read_instance_info = function()
 	{
 		var o = {}; //new doabc_instance_info();
 		var cnt;
@@ -1358,7 +1352,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_class_info = this.read_class_info = function $doABCReader_$read_class_info()
+	var read_class_info = this.read_class_info = function()
 	{
 		var o = {}; //new doabc_class_info();
 		o.cinit = read_u30();
@@ -1372,7 +1366,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_script_info = this.read_script_info = function $doABCReader_$read_script_info()
+	var read_script_info = this.read_script_info = function()
 	{
 		var o = {}; //new doabc_script_info();
 		o.init = read_u30();
@@ -1386,7 +1380,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_method_body_info = this.read_method_body_info = function $doABCReader_$read_method_body_info()
+	var read_method_body_info = this.read_method_body_info = function()
 	{
 		var o = {}; //new doabc_method_body_info();
 		var cnt;
@@ -1418,7 +1412,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_option_detail = this.read_option_detail = function $doABCReader_$read_option_detail()
+	var read_option_detail = this.read_option_detail = function()
 	{
 		var o = {}; //new doabc_option_detail();
 		o.val = read_u30();
@@ -1427,7 +1421,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_item_info = this.read_item_info = function $doABCReader_$read_item_info()
+	var read_item_info = this.read_item_info = function()
 	{
 		var o = {}; //new doabc_item_info();
 		o.key = read_u30();
@@ -1436,7 +1430,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_exception_info = this.read_exception_info = function $doABCReader_$read_exception_info()
+	var read_exception_info = this.read_exception_info = function()
 	{
 		var o = {}; //new doabc_exception_info();
 		o.from = read_u30();
@@ -1448,7 +1442,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_traits_info = this.read_traits_info = function $doABCReader_$read_traits_info()
+	var read_traits_info = this.read_traits_info = function()
 	{
 		var o = {}; //new doabc_traits_info();
 		o.name = read_u30();
@@ -1492,7 +1486,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_trait_slot = this.read_trait_slot = function $doABCReader_$read_trait_slot()
+	var read_trait_slot = this.read_trait_slot = function()
 	{
 		var o = {}; //new doabc_trait_slot();
 		o.slot_id = read_u30();
@@ -1506,7 +1500,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_trait_class = this.read_trait_class = function $doABCReader_$read_trait_class()
+	var read_trait_class = this.read_trait_class = function()
 	{
 		var o = {}; //new doabc_trait_class();
 		o.slot_id = read_u30();
@@ -1515,7 +1509,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_trait_function = this.read_trait_function = function $doABCReader_$read_trait_function()
+	var read_trait_function = this.read_trait_function = function()
 	{
 		var o = {}; //new doabc_trait_function();
 		o.slot_id = read_u30();
@@ -1524,7 +1518,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_trait_method = this.read_trait_method = function $doABCReader_$read_trait_method()
+	var read_trait_method = this.read_trait_method = function()
 	{
 		var o = {}; //new doabc_trait_method();
 		o.disp_id = read_u30();
@@ -1533,7 +1527,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_multiname_info_QName = this.read_multiname_info_QName = function $doABCReader_$read_multiname_info_QName()
+	var read_multiname_info_QName = this.read_multiname_info_QName = function()
 	{
 		var o = {}; //new doabc_multiname_info_QName();
 		o.ns = read_u30();
@@ -1542,7 +1536,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_multiname_info_RTQName = this.read_multiname_info_RTQName = function $doABCReader_$read_multiname_info_RTQName()
+	var read_multiname_info_RTQName = this.read_multiname_info_RTQName = function()
 	{
 		var o = {}; //new doabc_multiname_info_RTQName();
 		o.name = read_u30();
@@ -1550,14 +1544,14 @@ function doABCReader(bytes)
 	};
 
 
-	var read_multiname_info_RTQNameL = this.read_multiname_info_RTQNameL = function $doABCReader_$read_multiname_info_RTQNameL()
+	var read_multiname_info_RTQNameL = this.read_multiname_info_RTQNameL = function()
 	{
 		// This kind has no associated data.
 		return {}; //new doabc_multiname_info_RTQNameL();
 	};
 
 
-	var read_multiname_info_Multiname = this.read_multiname_info_Multiname = function $doABCReader_$read_multiname_info_Multiname()
+	var read_multiname_info_Multiname = this.read_multiname_info_Multiname = function()
 	{
 		var o = {}; //new doabc_multiname_info_Multiname();
 		o.name = read_u30();
@@ -1566,7 +1560,7 @@ function doABCReader(bytes)
 	};
 
 
-	var read_multiname_info_MultinameL = this.read_multiname_info_MultinameL = function $doABCReader_$read_multiname_info_MultinameL()
+	var read_multiname_info_MultinameL = this.read_multiname_info_MultinameL = function()
 	{
 		var o = {}; //new doabc_multiname_info_MultinameL();
 		o.ns_set = read_u30();
@@ -1763,114 +1757,64 @@ const AS3_OP_SETPROPERTY = 0x61;
 const AS3_OP_GETPROPERTY = 0x66;
 const AS3_OP_MODULO = 0xa4;
 
-var UTF8 = {
-	decode: function(str)
+function utf8_decode(str)
+{
+	var a = [];
+	for (var i = 0, len = str.length; i < len; ++i)
 	{
-		var a = [];
-		for (var i = 0, len = str.length; i < len; ++i)
+		var cc = str.charCodeAt(i);
+		if (cc > 255) { throw new Error("Illegal character: must be in range [0; 255]: " + cc); }
+		if (cc < 0x80)
 		{
-			var cc = str.charCodeAt(i);
-			if (cc > 255) { throw new Error("Illegal character: must be in range [0; 255]: " + cc); }
-			if (cc < 0x80)
-			{
-				a.push(str.charAt(i));
-				continue;
-			}
-			var n = 0;
-			var num_bytes = 0;
-			// 110xxxxx 10xxxxxx
-			if ((cc & 0xe0) === 0xc0)
-			{
-				num_bytes = 1;
-				n = cc & 0x1f;
-			}
-			// 1110xxxx 10xxxxxx 10xxxxxx
-			else if ((cc & 0xf0) === 0xe0)
-			{
-				num_bytes = 2;
-				n = cc & 0x0f;
-			}
-			// Other is not supported, because we have only 16 bits for code points.
-			// 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
-			// 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
-			// 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
-			else
-			{
-				throw new Error("Code point too big.");
-			}
-			var j = i + 1, j_end = j + num_bytes;
-			if (j_end > len)
-			{
-				throw new Error("Premature end of input string.");
-			}
-			for (; j !== j_end; ++j)
-			{
-				cc = str.charCodeAt(j);
-				if (cc > 255)
-				{
-					throw new Error("Illegal character: must be in range [0; 255]: " + cc);
-				}
-				if ((cc & 0xc0) !== 0x80)
-				{
-					throw new Error("Illegal character: high bits must be 10: " + cc
-						+ " (0x" + hex_pad(cc, cc < 0x100 ? 2 : 4) + ", " + bin_pad(cc, cc < 0x100 ? 8 : 16) + ")");
-				}
-				n <<= 6;
-				n |= cc & 0x3f;
-			}
-			a.push(String.fromCharCode(n));
-			i = j_end - 1;
+			a.push(str.charAt(i));
+			continue;
 		}
-		return a.join("");
-	},
-
-
-	encode: function(str)
-	{
-		var a = [];
-		for (var i = 0, len = str.length; i !== len; ++i)
+		var n = 0;
+		var num_bytes = 0;
+		// 110xxxxx 10xxxxxx
+		if ((cc & 0xe0) === 0xc0)
 		{
-			var cc = str.charCodeAt(i);
-			// 0xxxxxxx
-			if (cc < 0x80)
-			{
-				a.push(str.charAt(i));
-				continue;
-			}
-			var n, num_bytes;
-			// 110xxxxx 10xxxxxx
-			if (cc < 0x0800)
-			{
-				num_bytes = 1;
-				n = 0xc0 | ((cc >> 6) & 0x1f);
-				cc <<= 8;
-			}
-			// 1110xxxx 10xxxxxx 10xxxxxx
-			else if (cc < 0x10000)
-			{
-				num_bytes = 2;
-				n = 0xe0 | ((cc >> 12) & 0x0f);
-				cc <<= 2;
-			}
-			// Other is not supported, because we have only 16 bits for code points.
-			// 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
-			// 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
-			// 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
-			else
-			{
-				throw new Error("Code point too big.");
-			}
-			a.push(String.fromCharCode(n));
-			for (var j = 0; j !== num_bytes; ++j)
-			{
-				n = 0x80 | ((cc >> 8) & 0x3f);
-				a.push(String.fromCharCode(n));
-				cc <<= 6;
-			}
+			num_bytes = 1;
+			n = cc & 0x1f;
 		}
-		return a.join("");
+		// 1110xxxx 10xxxxxx 10xxxxxx
+		else if ((cc & 0xf0) === 0xe0)
+		{
+			num_bytes = 2;
+			n = cc & 0x0f;
+		}
+		// Other is not supported, because we have only 16 bits for code points.
+		// 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
+		// 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+		// 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+		else
+		{
+			throw new Error("Code point too big.");
+		}
+		var j = i + 1, j_end = j + num_bytes;
+		if (j_end > len)
+		{
+			throw new Error("Premature end of input string.");
+		}
+		for (; j !== j_end; ++j)
+		{
+			cc = str.charCodeAt(j);
+			if (cc > 255)
+			{
+				throw new Error("Illegal character: must be in range [0; 255]: " + cc);
+			}
+			if ((cc & 0xc0) !== 0x80)
+			{
+				throw new Error("Illegal character: high bits must be 10: " + cc);
+			}
+			n <<= 6;
+			n |= cc & 0x3f;
+		}
+		a.push(String.fromCharCode(n));
+		i = j_end - 1;
 	}
-};
+	return a.join("");
+}
 
 
 
@@ -1915,16 +1859,7 @@ function refresh_signature_func(ctx)
 
 	var stopwatch = new Stopwatch();
 	stopwatch.start();
-	var rc;
-	var decoders = [decode1, decode2];
-	for (var i = 0, len = decoders.length; ! rc && i !== len; ++i)
-	{
-		try { rc = decoders[i](df); }
-		catch (x)
-		{
-			//log("YoutubeSwf.js::refresh_signature_func: decoders[" + i + "] failed: " + (x.message || x) + "\n" + (x.stack || new Error().stack));
-		}
-	}
+	var rc = decode(df);
 	if ( ! rc) { return; }
 	var code = [
 		"/* Not encoded. */",
@@ -1958,149 +1893,6 @@ function refresh_signature_func(ctx)
 }
 
 
-function decode1(df)
-{
-	var rc = {timestamp: -1, calls: []};
-
-	var cp = df.constant_pool;
-	var inst_idx = -1;
-	for (var i = 0, len = df.instances.length; i !== len; ++i) 
-	{
-		var ii = df.instances[i];
-		if (cp.multinames[ii.name].kind !== doabc_CONSTANT_QName) { continue; }
-		if (cp.resolve_multiname(ii.name) === "com.google.youtube.util.SignatureDecipher")
-		{
-			inst_idx = i;
-			break;
-		}
-	}
-	if (inst_idx === -1) { return; }
-	var index_by_name = {"decipher": null, "clone": null, "swap": null, "reverse": null};
-	var name_by_index = {};
-	var cls = df.classes[inst_idx];
-	for (var i = 0, len = cls.traits.length; i !== len; ++i)
-	{
-		var ti = cls.traits[i];
-		if (ti.kind !== doabc_Trait_Method || cp.multinames[ti.name].kind !== doabc_CONSTANT_QName) { continue; }
-		var s_name = cp.resolve_multiname(ti.name);
-		var map_name, map_idx;
-		if (/^(clone|swap|reverse)_(\d+)$/.test(s_name))
-		{
-			map_name = RegExp.$1;
-			map_idx = ti.name;
-			rc.timestamp = parseInt(RegExp.$2);
-		}
-		else if ("decipher" === s_name)
-		{
-			map_name = s_name;
-			map_idx = ti.trait.method;
-		}
-		if (!map_name) { continue; }
-		if (index_by_name[map_name] != null) { return; }
-		index_by_name[map_name] = map_idx;
-		name_by_index[map_idx] = map_name;
-	}
-	var m_idx = index_by_name["decipher"], mb = null;
-	if (!df.method_bodies.some(function(o){return o.method === m_idx && (mb = o);})) { return; }
-	var dar = new doABCReader(mb.code);
-	var state = {states: null, idx: -1};
-	function advance_if(cond) { return cond ? ++state.idx : false; }
-	function switch_if(cond, states, idx /*= 0*/)
-	{
-		if (!cond) { return false; }
-		state.states = states;
-		state.idx = idx || 0;
-		return true;
-	}
-	function done_if(cond) { return cond ? null : false; }
-	var PS_START = [
-		AS3_OP_GETLOCAL_0,
-		AS3_OP_PUSHSCOPE,
-		AS3_OP_GETLOCAL_1,
-		function(op) { return advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(dar.read_u30()).length === 0); },
-		function(op) {
-			return switch_if(op === AS3_OP_CALLPROPERTY
-				&& cp.resolve_multiname(dar.read_u30()) === "http://adobe.com/AS3/2006/builtin.split"
-				&& dar.read_u30() === 1, PS_STORE_AND_BRANCH);
-		}
-	];
-	var PS_STORE_AND_BRANCH = [
-		function(op) { return advance_if(op === AS3_OP_COERCE && cp.resolve_multiname(dar.read_u30()) === "Array"); },
-		AS3_OP_SETLOCAL_2,
-		function(op) {
-			if (op === AS3_OP_FINDPROPSTRICT) {
-				var name = name_by_index[dar.read_u30()];
-				return switch_if(name === "swap", PS_SWAP)
-					|| switch_if(name === "clone", PS_CLONE)
-					|| switch_if(name === "reverse", PS_REVERSE);
-			}
-			return advance_if(op === AS3_OP_GETLOCAL_2);
-		},
-		function(op) { return advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(dar.read_u30()).length === 0); },
-		function(op) {
-			return advance_if(op === AS3_OP_CALLPROPERTY
-				&& cp.resolve_multiname(dar.read_u30()) === "http://adobe.com/AS3/2006/builtin.join"
-				&& dar.read_u30() === 1);
-		},
-		function(op) { return done_if(op === AS3_OP_RETURNVALUE); }
-	];
-	var PS_SWAP = [
-		AS3_OP_GETLOCAL_2,
-		function(op) { return advance_if(op === AS3_OP_PUSHBYTE && rc.calls.push( {name: "swap", arg: dar.read_u8()} )); },
-		function(op) {
-			return switch_if(op === AS3_OP_CALLPROPERTY
-				&& dar.read_u30() === index_by_name["swap"]
-				&& dar.read_u30() === 2, PS_STORE_AND_BRANCH);
-		}
-	];
-	var PS_CLONE = [
-		AS3_OP_GETLOCAL_2,
-		function(op) { return advance_if(op === AS3_OP_PUSHBYTE && rc.calls.push( {name: "clone", arg: dar.read_u8()} )); },
-		function(op) {
-			return switch_if(op === AS3_OP_CALLPROPERTY
-				&& dar.read_u30() === index_by_name["clone"]
-				&& dar.read_u30() === 2, PS_STORE_AND_BRANCH);
-		}
-	];
-	var PS_REVERSE = [
-		function(op) { return advance_if(op === AS3_OP_GETLOCAL_2 && rc.calls.push( {name: "reverse"} )); },
-		function(op) {
-			return switch_if(op === AS3_OP_CALLPROPERTY
-				&& dar.read_u30() === index_by_name["reverse"]
-				&& dar.read_u30() === 1, PS_STORE_AND_BRANCH);
-		}
-	];
-
-	state.states = PS_START;
-	state.idx = 0;
-	var done = false;
-	while ( ! done && ! dar.eof())
-	{
-		var op = dar.read_u8();
-		var sv = states[idx];
-		var res;
-		switch (typeof(sv))
-		{
-			case "number":
-				res = advance_if(op === sv);
-				break;
-			case "function":
-				res = sv(op);
-				break;
-			default:
-				throw new Error("Invalid state type: " + typeof(sv));
-		}
-		if ( ! res)
-		{
-			done = res === null;
-			break;
-		}
-	}
-	return done ? rc : null;
-}
-
-
-// 2014-07-15.
 function Decoder(file, reader, states, idx /*= 0*/)
 {
 	this.result = {timestamp: -1, calls: []};
@@ -2114,15 +1906,16 @@ Decoder.prototype = {
 		while ( ! done && ! this.reader.eof())
 		{
 			var res;
-			var op = this.reader.read_u8();
+			var o = this.parse_op();
+			if ( ! o) { break; }
 			var sv = this.states[this.idx];
 			switch (typeof(sv))
 			{
 				case "number":
-					res = this.advance_if(op === sv);
+					res = this.advance_if(o.op === sv);
 					break;
 				case "function":
-					res = sv.call(this, op);
+					res = sv.call(this, o.op, o);
 					break;
 				default:
 					throw new Error("Invalid state type: " + typeof(sv));
@@ -2135,6 +1928,58 @@ Decoder.prototype = {
 		}
 		return done ? this.result : null;
 	},
+	parse_op: function()
+	{
+		var o = {};
+		var dar = this.reader;
+		switch (o.op = dar.read_u8())
+		{
+			case AS3_OP_COERCE_S:
+			case AS3_OP_GETLOCAL_0:
+			case AS3_OP_GETLOCAL_1:
+			case AS3_OP_GETLOCAL_2:
+			case AS3_OP_GETLOCAL_3:
+			case AS3_OP_MODULO:
+			case AS3_OP_PUSHSCOPE:
+			case AS3_OP_SETLOCAL_1:
+			case AS3_OP_SETLOCAL_2:
+			case AS3_OP_SETLOCAL_3:
+			case AS3_OP_RETURNVALUE:
+			case AS3_OP_RETURNVOID:
+				break;
+			case AS3_OP_PUSHBYTE:
+				o.value = dar.read_u8();
+				break;
+			case AS3_OP_PUSHSHORT:
+				o.value = dar.read_u30();
+				break;
+			case AS3_OP_COERCE:
+			case AS3_OP_FINDPROPERTY:
+			case AS3_OP_FINDPROPSTRICT:
+			case AS3_OP_GETLEX:
+			case AS3_OP_GETLOCAL:
+			case AS3_OP_GETPROPERTY:
+			case AS3_OP_PUSHINT:
+			case AS3_OP_PUSHSTRING:
+			case AS3_OP_PUSHUINT:
+			case AS3_OP_SETLOCAL:
+			case AS3_OP_SETPROPERTY:
+				o.index = dar.read_u30();
+				break;
+			case AS3_OP_IFTRUE:
+				o.offset = dar.read_s24();
+				break;
+			case AS3_OP_CALLPROPERTY:
+			case AS3_OP_CALLPROPVOID:
+			case AS3_OP_CONSTRUCTPROP:
+				o.index = dar.read_u30();
+				o.arg_count = dar.read_u30();
+				break;
+			default:
+				return null;
+		}
+		return o;
+	},
 	advance_if: function(cond) { return cond ? ++this.idx : false; },
 	switch_if: function(cond, states, idx /*= 0*/)
 	{
@@ -2151,7 +1996,7 @@ Decoder.prototype = {
 	},
 };
 
-function decode2(df)
+function decode(df)
 {
 	var cp = df.constant_pool;
 	var inst_idx = -1;
@@ -2184,19 +2029,20 @@ function decode2(df)
 	}
 	if ( ! mb_decipher) { return; }
 
-	var ctx = {ts_var_idx: -1, inst_var_idx: -1, cls_name_idx: -1, cls_idx: -1, func_name_idx: -1, name_by_index: {}, call_arg: null};
+	var ctx = {cls_name_idx: -1, func_name_idx: -1, name_by_index: {}, call_arg: null};
 	var PS_TIMESTAMP = [
-		AS3_OP_GETLOCAL_0,
-		AS3_OP_PUSHSCOPE,
-		function(op) { return this.advance_if(op === AS3_OP_FINDPROPERTY && (ctx.ts_var_idx = this.reader.read_u30())); },
-		function(op) {
-			return this.advance_if(op === AS3_OP_PUSHSHORT && (this.result.timestamp = this.reader.read_u30()))
-				|| this.advance_if(op === AS3_OP_PUSHINT && (this.result.timestamp = cp.resolve_integer(this.reader.read_u30())))
-				|| this.advance_if(op === AS3_OP_PUSHUINT && (this.result.timestamp = cp.resolve_uinteger(this.reader.read_u30())));
+		function(op, o) {
+			return op !== AS3_OP_FINDPROPERTY
+				|| cp.resolve_multiname(o.index) !== "TIMESTAMP"
+				|| this.advance_if(true);
+		},
+		function(op, o) {
+			return this.advance_if(op === AS3_OP_PUSHSHORT && (this.result.timestamp = o.value, 1))
+				|| this.advance_if(op === AS3_OP_PUSHINT && (this.result.timestamp = cp.resolve_integer(o.index), 1))
+				|| this.advance_if(op === AS3_OP_PUSHUINT && (this.result.timestamp = cp.resolve_uinteger(o.index), 1));
 		},
-		function(op) { return this.advance_if(op === AS3_OP_SETPROPERTY && this.reader.read_u30() === ctx.ts_var_idx); },
 		function(op) {
-			if (op !== AS3_OP_RETURNVOID) { return false; }
+			if (op !== AS3_OP_SETPROPERTY) { return false; }
 			this.reset(new doABCReader(mb_decipher.code), PS_DECIPHER);
 			return true;
 		}
@@ -2204,39 +2050,27 @@ function decode2(df)
 	var PS_DECIPHER = [
 		AS3_OP_GETLOCAL_0,
 		AS3_OP_PUSHSCOPE,
-		function(op) { return this.advance_if(op === AS3_OP_GETLEX && (ctx.inst_var_idx = this.reader.read_u30())); },
-		function(op) { return this.advance_if(op === AS3_OP_IFTRUE && this.reader.read_s24()); },
-		function(op) { return this.advance_if(op === AS3_OP_FINDPROPERTY && this.reader.read_u30() === ctx.inst_var_idx); },
-		function(op) { return this.advance_if(op === AS3_OP_FINDPROPSTRICT && (ctx.cls_name_idx = this.reader.read_u30())); },
-		function(op) {
-			return this.advance_if(op === AS3_OP_CONSTRUCTPROP
-				&& this.reader.read_u30() === ctx.cls_name_idx
-				&& this.reader.read_u30() === 0);
-		},
-		function(op) { return this.advance_if(op === AS3_OP_SETPROPERTY && this.reader.read_u30() === ctx.inst_var_idx); },
-		function(op) { return this.advance_if(op === AS3_OP_GETLEX && this.reader.read_u30() === ctx.inst_var_idx); },
+		AS3_OP_GETLEX,
+		AS3_OP_IFTRUE,
+		AS3_OP_FINDPROPERTY,
+		function(op, o) { return this.advance_if(op === AS3_OP_FINDPROPSTRICT && (ctx.cls_name_idx = o.index)); },
+		AS3_OP_CONSTRUCTPROP,
+		AS3_OP_SETPROPERTY,
+		AS3_OP_GETLEX,
 		AS3_OP_GETLOCAL_1,
-		function(op) { return this.advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(this.reader.read_u30()).length === 0); },
-		function(op) {
+		function(op, o) { return this.advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(o.index).length === 0); },
+		function(op, o) {
 			return this.advance_if(op === AS3_OP_CALLPROPERTY
-				&& cp.resolve_multiname(this.reader.read_u30()) === "http://adobe.com/AS3/2006/builtin.split"
-				&& this.reader.read_u30() === 1);
+				&& cp.resolve_multiname(o.index) === "http://adobe.com/AS3/2006/builtin.split");
 		},
-		function(op) {
+		function(op, o) { return this.advance_if(op === AS3_OP_CALLPROPERTY && (ctx.func_name_idx = o.index)); },
+		function(op, o) { return this.advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(o.index).length === 0); },
+		function(op, o) {
 			return this.advance_if(op === AS3_OP_CALLPROPERTY
-				&& (ctx.func_name_idx = this.reader.read_u30())
-				&& this.reader.read_u30() === 1);
-		},
-		function(op) { return this.advance_if(op === AS3_OP_PUSHSTRING && cp.resolve_string(this.reader.read_u30()).length === 0); },
-		function(op) {
-			return this.advance_if(op === AS3_OP_CALLPROPERTY
-				&& cp.resolve_multiname(this.reader.read_u30()) === "http://adobe.com/AS3/2006/builtin.join"
-				&& this.reader.read_u30() === 1);
+				&& cp.resolve_multiname(o.index) === "http://adobe.com/AS3/2006/builtin.join");
 		},
 		function(op) {
 			if (op !== AS3_OP_RETURNVALUE) { return false; }
-			ctx.cls_idx = this.file.get_instance_index_by_name_index(ctx.cls_name_idx);
-			if (ctx.cls_idx === -1) { return false; }
 			var mb = this.file.get_method_body_by_name_index(ctx.cls_name_idx, ctx.func_name_idx);
 			if ( ! mb) { return false; }
 			this.reset(new doABCReader(mb.code), PS_FUNC);
@@ -2249,94 +2083,41 @@ function decode2(df)
 	];
 	var PS_FUNC_BRANCH = [
 		function(op) {
-			return this.advance_if(op === AS3_OP_GETLOCAL_1)
-				|| this.switch_if(op === AS3_OP_GETLOCAL_0, PS_FUNC_CALL);
+			return this.switch_if(op === AS3_OP_GETLOCAL_0, PS_FUNC_CALL)
+				|| this.advance_if(op === AS3_OP_GETLOCAL_1);
 		},
 		function(op) { return this.done_if(op === AS3_OP_RETURNVALUE); }
 	];
 	var PS_FUNC_CALL = [
 		AS3_OP_GETLOCAL_1,
-		function(op) { return this.advance_if(op === AS3_OP_PUSHBYTE && (ctx.call_arg = this.reader.read_u8(), 1)); },
-		function(op) {
+		function(op, o) { return this.advance_if(op === AS3_OP_PUSHBYTE && (ctx.call_arg = o.value, 1)); },
+		function(op, o) {
 			if (op !== AS3_OP_CALLPROPERTY) { return false; }
-			var func_name_idx = this.reader.read_u30();
-			if (this.reader.read_u30() !== 2) { return false; }
+			var func_name_idx = o.index;
 			var func_name = ctx.name_by_index[func_name_idx];
 			if ( ! func_name)
 			{
 				var mb = this.file.get_method_body_by_name_index(ctx.cls_name_idx, func_name_idx);
 				if ( ! mb) { return false; }
-				var PS_CLONE = [
-					AS3_OP_GETLOCAL_0,
-					AS3_OP_PUSHSCOPE,
-					AS3_OP_GETLOCAL_1,
-					AS3_OP_GETLOCAL_2,
-					function(op) {
-						return this.advance_if(op === AS3_OP_CALLPROPERTY
-							&& cp.resolve_multiname(this.reader.read_u30()) === "http://adobe.com/AS3/2006/builtin.slice"
-							&& this.reader.read_u30() === 1);
-					},
-					function(op) { return this.done_if(op === AS3_OP_RETURNVALUE); }
-				];
-				var PS_REVERSE = [
+				var PS_TEST = [
 					AS3_OP_GETLOCAL_0,
 					AS3_OP_PUSHSCOPE,
 					AS3_OP_GETLOCAL_1,
 					function(op) {
-						return this.advance_if(op === AS3_OP_CALLPROPVOID
-							&& cp.resolve_multiname(this.reader.read_u30()) === "http://adobe.com/AS3/2006/builtin.reverse"
-							&& (this.reader.read_u30(), 1));
-					},
-					AS3_OP_GETLOCAL_1,
-					function(op) { return this.done_if(op === AS3_OP_RETURNVALUE); }
+						return this.done_if((op === AS3_OP_GETLOCAL_2 && (this.result = "clone"))
+							|| (op === AS3_OP_CALLPROPVOID && (this.result = "reverse"))
+							|| (op === AS3_OP_PUSHBYTE && (this.result = "swap")));
+					}
 				];
-				var PS_SWAP = [
-					AS3_OP_GETLOCAL_0,
-					AS3_OP_PUSHSCOPE,
-					AS3_OP_GETLOCAL_1,
-					function(op) { return this.advance_if(op === AS3_OP_PUSHBYTE && (this.reader.read_u8(), 1)); },
-					function(op) { return this.advance_if(op === AS3_OP_GETPROPERTY && this.reader.read_u30()); },
-					function(op) { return op === AS3_OP_COERCE_S || this.advance_if(op === AS3_OP_SETLOCAL_3); },
-					AS3_OP_GETLOCAL_1,
-					AS3_OP_GETLOCAL_2,
-					AS3_OP_GETLOCAL_1,
-					function(op) {
-						return this.advance_if(op === AS3_OP_GETPROPERTY
-							&& cp.resolve_multiname(this.reader.read_u30()) === "length");
-					},
-					AS3_OP_MODULO,
-					function(op) { return this.advance_if(op === AS3_OP_GETPROPERTY && this.reader.read_u30()); },
-					function(op) { return op === AS3_OP_COERCE_S || this.advance_if(op === AS3_OP_SETLOCAL && (this.reader.read_u30(), 1)); },
-					AS3_OP_GETLOCAL_1,
-					function(op) { return this.advance_if(op === AS3_OP_PUSHBYTE && (this.reader.read_u8(), 1)); },
-					function(op) { return this.advance_if(op === AS3_OP_GETLOCAL && (this.reader.read_u30(), 1)); },
-					function(op) { return this.advance_if(op === AS3_OP_SETPROPERTY && this.reader.read_u30()); },
-					AS3_OP_GETLOCAL_1,
-					AS3_OP_GETLOCAL_2,
-					AS3_OP_GETLOCAL_3,
-					function(op) { return this.advance_if(op === AS3_OP_SETPROPERTY && this.reader.read_u30()); },
-					AS3_OP_GETLOCAL_1,
-					function(op) { return this.done_if(op === AS3_OP_RETURNVALUE); }
-				];
-				var map = [
-					{state: PS_CLONE, name: "clone"},
-					{state: PS_REVERSE, name: "reverse"},
-					{state: PS_SWAP, name: "swap"}
-				];
-				map.some(function(o){
-					var d = new Decoder(df, new doABCReader(mb.code), o.state);
-					try { return d.run() && (func_name = o.name); }
-					catch (ignore) {}
-					return false;
-				}, this);
+				var d = new Decoder(df, new doABCReader(mb.code), PS_TEST);
+				if (d.run()) { func_name = d.result; }
 				if ( ! func_name) { return false; }
 				ctx.name_by_index[func_name_idx] = func_name;
 			}
 			this.result.calls.push({name: func_name, arg: ctx.call_arg});
 			return this.advance_if(true);
 		},
-		function(op) { return this.advance_if(op === AS3_OP_COERCE && cp.resolve_multiname(this.reader.read_u30()) === "Array"); },
-		function(op) { return this.switch_if(op === AS3_OP_SETLOCAL_1, PS_FUNC_BRANCH); }
+		function(op) { return op === AS3_OP_COERCE || this.switch_if(op === AS3_OP_SETLOCAL_1, PS_FUNC_BRANCH); }
 	];
 
 	var d = new Decoder(df, new doABCReader(mb_cinit.code), PS_TIMESTAMP);
