@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -89,6 +89,11 @@ FlashGotGalleryBuilder.prototype = {
       if(this.sandbox) {
         try {
           this.expressions = Components.utils.evalInSandbox(this.prefs.getCharPref("buildGallery.expressions"), this.sandbox);
+          // Gecko 31: functions and some other types are not visible by default anymore.
+          // See https://developer.mozilla.org/en-US/docs/Xray_vision
+          if (this.expressions && Components.utils.waiveXrays) {
+            this.expressions = Components.utils.waiveXrays(this.expressions);
+          }
         } catch(ex) {}
         if(typeof(this.expressions)!="object" || !this.expressions) this.expressions = {};
       } else {
@@ -306,12 +311,50 @@ FlashGotGalleryBuilder.prototype = {
   }
 ,
   createExpression: function(text) {
-    expr = { text: text };
-    try {
-      expr.func = Components.utils.evalInSandbox("_f = function() {\n" + text + "\n}", this.sandbox);
-      expr.err = null;
-    } catch(err) {
-      expr.err = err;
+    function create_in_sandbox(text) {
+      var expr = { text: text };
+      try {
+        // Function() creates a named function ("anonymous"),
+        // but gives "better" syntax errors than eval() except
+        // when it's an unmatched closing curly bracket "}".
+        // In the latter case, it gives the line number of the
+        // preceeding parsed contruct, i.e. the parser eats
+        // whitespace without incrementing the line number,
+        // e.g. "return 123;\n}" reports the "virtual" line
+        // number 1 and not 2, and "return 123;\n\n\n\n\n}"
+        // reports the "virtual" line number 1 as well.
+        expr.func = new Function("", text); //eval("(function(){" + text + "\n});");
+        expr.err = null;
+      } catch(err) {
+        expr.err = err;
+      }
+      return expr;
+    }
+    var expr = Components.utils.evalInSandbox(""
+      + "var $f = " + create_in_sandbox + "\n;"
+      + "var $t = String(" + text.toSource() + "\n);" // (new String("abc")) -> "abc".
+      + "$f($t);"
+      , this.sandbox);
+    // Gecko 31: functions and some other types are not visible by default anymore.
+    // See https://developer.mozilla.org/en-US/docs/Xray_vision
+    if (expr && Components.utils.waiveXrays) { expr = Components.utils.waiveXrays(expr); }
+    // Get the source line number and text.
+    // Depends on how buggy the JS parser is - see the comment in the try block.
+    var err = expr && expr.err;
+    if (err && err.lineNumber != null && arguments.callee.caller != null && arguments.callee.caller !== arguments.callee) {
+      // Compile/eval something that gives a syntax error in line 1.
+      var tmpExpr = this.createExpression(":");
+      if (tmpExpr && tmpExpr.err && tmpExpr.err.lineNumber != null) {
+        var lines = text.split(/\r\n|\r|\n/);
+        var sourceLineNumber = Math.min(lines.length - 1, err.lineNumber - tmpExpr.err.lineNumber);
+        var sourceLine = lines[sourceLineNumber];
+        // Ignore trailing empty lines, e.g. "-\n\n\n".
+        for (var i = lines.length; ! sourceLine.length && i-- > 0; ) {
+          sourceLine = lines[sourceLineNumber = i];
+        }
+        expr.errLineNumber = sourceLineNumber + 1;
+        expr.errLineText = sourceLine;
+      }
     }
     return expr;
   }
@@ -389,19 +432,20 @@ FlashGotGalleryBuilder.prototype = {
   }
 ,
   _hilightErrors: function() {
-    const exprList=this.exprListBox;
-    exprList.style.background="white";
-    const ee=this.expressions;
-    var item,expr;
-    for(var j=exprList.getRowCount(); j-->0;) {
-      item=exprList.getItemAtIndex(j);
-      expr=ee[item.label];
-      if(expr) {
-        item.style.color=expr.err?"red":"black";
-        if(item.selected) {
-          document.getElementById("flashgotGB-expr-error-text").value=expr.err;
-          this.exprTextBox.style.color=item.style.color;
-        }
+    const exprList = this.exprListBox;
+    exprList.style.background = "white";
+    const item = exprList.selectedItem;
+    if ( ! item) { return; }
+    const expr = this.expressions[item.label];
+    if ( ! expr) { return; }
+    const err = expr.err;
+    this.exprTextBox.style.color = item.style.color = err ? "red" : "black";
+    const errTextBox = document.getElementById("flashgotGB-expr-error-text");
+    errTextBox.value = err;
+    if (err && expr.errLineNumber) {
+      errTextBox.value += "\nLine: " + expr.errLineNumber;
+      if (expr.errLineText) {
+        errTextBox.value += ":\n" + expr.errLineText + "\n";
       }
     }
   }
@@ -422,11 +466,30 @@ function FlashGotGalleryHTML(builder) {
      exprNames[exprNames.length]=match[1];
   }
   const ee=builder.expressions;
-  
+
   for(name in ee) {
     exprNames[exprNames.length]=name;
     expr=ee[name];
-    this.env[name] = expr.func ? expr.func : function() { throw new Error("["+name+"()]"+" not implemented!"); };
+    this.env[name] = expr.func
+      // FF 32.0.1: transfering (assigning) a function from one sandbox
+      // to another doesn't work: typeof works, but calling it throws
+      // "Permission denied to access object", and accessing its properties
+      // (either explicitly or implicitly, e.g. in a string concatenation)
+      // throws "Permission denied to access property '...'".
+      // Creating a proxy function doesn't work either - |arguments| is
+      // empty:
+      //   function create_proxy_func(expr) {
+      //     return function(){ return expr.func.apply(null, arguments); };
+      //   }
+      //   this.env[name] = expr.func ? create_proxy_func(expr) : ...;
+      // So we have 2 options:
+      // 1) create a new function in this sandbox.
+      // 2) use one sandbox for everything: loading the functions from prefs
+      // (FlashGotGalleryBuilder.onload), creating them from the editor text
+      // (FlashGotGalleryBuilder.createExpression), and calling them
+      // (FlashGotGalleryHTML.evalExpressions).
+      ? new Function("", expr.text)
+      : function() { throw new Error("["+name+"()]"+" not implemented!"); };
   }
   
   exprNames=exprNames.sort();
@@ -480,8 +543,8 @@ FlashGotGalleryHTML.prototype = {
     return "\n</div></body>";
   }
 ,
-  _eval: function(ctx, parm) {
-    return Components.utils.evalInSandbox("_func(" + parm + ")", ctx);
+  _eval: function(ctx, name, parm) {
+    return Components.utils.evalInSandbox(name + "(" + parm + ")", ctx);
   }
 ,
   // evaluates macros and javascript functions
@@ -532,8 +595,7 @@ FlashGotGalleryHTML.prototype = {
       subst=match[0];
       if(expr && expr.func) {
         try {
-          ctx._func=expr.func;
-          res = this._eval(ctx, match[2]);
+          res = this._eval(ctx, name, match[2]);
           if(res != null && typeof(res) != "undefined") {
             subst=res;
           }
