@@ -61,9 +61,19 @@ var gFlashGot = {
     target.addEventListener("load", this.updateMediaStatus, true);
     
     target.addEventListener("mousedown", function(ev) {
-      if (!(ev.button <= 1 && gFlashGot._isContentEvent(ev))) return;
-      if (gFlashGot.mouseDown && gFlashGot.mouseDown.gesture === 1)
-        gFlashGotService.cursor(false);
+      if (gFlashGot.mouseDown) {
+        if (gFlashGot.mouseDown.gesture === 1) {
+          gFlashGotService.cursor(false);
+        }
+        gFlashGot.mouseDown = null;
+      }
+      
+      if (ev.button > 1 ||
+          ev.button === 1 && !gFlashGotService.getPref("gesture") ||
+          !gFlashGot._isContentEvent(ev)
+          ) {
+        return;
+      }
       
       gFlashGot.mouseDown = {
         x: ev.screenX,
@@ -243,7 +253,7 @@ var gFlashGot = {
   }
 ,
   browseHomePage: function() {
-    this.browse("http://flashgot.net");
+    this.browse("https://flashgot.net");
   }
 ,
   get hideIcons() {
@@ -676,28 +686,48 @@ var gFlashGot = {
     }
   },
   
-  prepareMediaMenu: function(menu) {
-    var menu_parsed = document.getElementById("flashgot-media-parsed-popup");
-    var menu_parsed_dash = document.getElementById("flashgot-media-parsed-dash-popup");
-    var mi_refresh_decode_signature_func = document.getElementById("flashgot-media-parsed-refresh_signature_func");
-    this._clearMenu(document.getElementById("flashgot-media-parsed-menu").nextSibling);
-    this._clearMenu(mi_refresh_decode_signature_func.nextSibling);
-    this._clearMenu(menu_parsed_dash.firstChild);
-    menu_parsed.parentNode.setAttribute("hidden", "true");
-    menu_parsed_dash.parentNode.setAttribute("hidden", "true");
-    mi_refresh_decode_signature_func.setAttribute("hidden", "true");
-    var m = this.media;
-    if (!(m && m.length)) return false;
-  
-    // YouTube parsed streams. We'll sort them before building the menu.
-    var parsed_items = [], parsed_dash_items = [];
+  prepareMediaMenu: function(menu, evt) {
+    const menu_parsed = document.getElementById("flashgot-media-parsed-popup");
+    const menu_parsed_dash = document.getElementById("flashgot-media-parsed-dash-popup");
+    const mi_refresh_decode_signature_func = document.getElementById("flashgot-media-parsed-refresh_signature_func");
+    mi_refresh_decode_signature_func.hidden = true;
+    const m = this.media;
+    if (!(m && m.length)) {
+      menu_parsed.parentNode.hidden = true;
+      menu_parsed_dash.parentNode.hidden = true;
+      this.clearMediaMenu();
+      return false;
+    }
+
+    // YouTube parsed streams.
+    // Map<string video_id, Group data> groups;
+    // struct Group {
+    //   string id; // Video ID.
+    //   string title; // Video title, goes to the menu label.
+    //   string link; // Video page URL, goes to clipboard along with the video title.
+    //   Array<menuitem> parsed_items;
+    //   Array<menuitem> parsed_dash_items;
+    //   menupopup menu_parsed; // |parsed_items| go here.
+    //   menupopup menu_parsed_dash; // |parsed_dash_items| go here.
+    // }
+    const groups = {};
+    // Dummy group for the non-grouping case.
+    var group = {
+      parsed_items: [],
+      parsed_dash_items: [],
+      menu_parsed: menu_parsed,
+      menu_parsed_dash: menu_parsed_dash
+    };
+    // Actually played (not parsed) streams.
+    const played_items = [];
+    var do_group = gFlashGotService.getPref("media.YouTube.group.enabled", false);
+    if ( ! do_group) { groups[""] = group; }
     for (var j = 0; j < m.length; j++) {
       var mo = m[j];
       // Special menu item, used for refreshing YouTube's signature decoding function.
       if (mo.MediaSniffer && mo.originalURL === mo.MediaSniffer.YOUTUBE_REFRESH_SIGNATURE_FUNC_URL) {
         var mi = mi_refresh_decode_signature_func;
-        mi.setAttribute("hidden", "false");
-        menu_parsed.parentNode.setAttribute("hidden", "false");
+        mi.hidden = false;
         mi["flashgot::Youtube"] = mo.Youtube;
         mi["flashgot::MediaSniffer"] = mo.MediaSniffer;
         mi["flashgot::yt_win"] = mo.yt_win;
@@ -705,54 +735,124 @@ var gFlashGot = {
       }
       var mi = this._createMediaMenuItem(mo);
       if (!mo.parsed) {
-        menu.appendChild(mi);
+        played_items.push(mi);
         continue;
       }
+      if (do_group) {
+        group = groups[mo.video_id];
+        if ( ! group) {
+          group = {
+            id: mo.video_id,
+            title: mo.title != null ? String(mo.title) : "Video " + mo.video_id,
+            link: mo.video_link,
+            parsed_items: [],
+            parsed_dash_items: [],
+            menu_parsed: menu_parsed,
+            menu_parsed_dash: menu_parsed_dash
+          };
+          groups[group.id] = group;
+        }
+      }
       mi["flashgot::sort_key"] = mo.sort_key;
       if (mo.yt_dash) {
         mi["flashgot::yt_dash"] = mo.yt_dash;
-        parsed_dash_items.push(mi);
+        group.parsed_dash_items.push(mi);
       } else {
-        parsed_items.push(mi);
+        group.parsed_items.push(mi);
       }
     }
-    if (parsed_items.length || parsed_dash_items.length) {
-      function mo_cmp(lhs, rhs) { return lhs["flashgot::sort_key"] - rhs["flashgot::sort_key"]; }
+
+    // Sort the groups by video title. Ignore empty groups.
+    const group_ids = Object.keys(groups)
+      .filter(function(id){var o = groups[id]; return o.parsed_items.length || o.parsed_dash_items.length;})
+      .sort(function(lhs, rhs){return groups[lhs].title.toLowerCase().localeCompare(groups[rhs].title.toLowerCase());});
+    do_group &= group_ids.length > 1;
+    if (evt.target !== menu && do_group) { return; }
+
+    menu_parsed.parentNode.hidden = true;
+    menu_parsed_dash.parentNode.hidden = true;
+    this.clearMediaMenu();
+    played_items.forEach(function(mi){menu.appendChild(mi);});
+
+    function mo_cmp(lhs, rhs) { return lhs["flashgot::sort_key"] - rhs["flashgot::sort_key"]; }
+    const group_title_max_length = gFlashGotService.getPref("media.YouTube.group.title_max_length", 40);
+    const dash_menu_label = menu_parsed_dash.parentNode.label;
+    group_ids.forEach(function(gid){
+      var group = groups[gid];
+      var parsed_items = group.parsed_items;
+      var parsed_dash_items = group.parsed_dash_items;
+      menu_parsed.parentNode.hidden = false;
+      if (do_group) {
+        // Non-DASH streams menu.
+        var gmenu = document.createElement("menu");
+        menu_parsed.appendChild(gmenu);
+        gmenu.setAttribute("label", group.title.length > group_title_max_length
+          ? group.title.substr(0, group_title_max_length) + "..." : group.title);
+        gmenu.setAttribute("tooltiptext", group.title + " (" + group.id + ")");
+        gmenu.appendChild(group.menu_parsed = document.createElement("menupopup"));
+        // Right-click to copy the video title and link to clipboard.
+        gmenu.setAttribute("flashgot_media_clipboard_text", group.title + (group.link ? " (" + group.link + ")" : ""));
+        gmenu.addEventListener("click", function(evt){
+          if (evt.button !== 2 || evt.target !== this) { return; }
+          evt.preventDefault();
+          evt.stopPropagation();
+          Components.classes["@mozilla.org/widget/clipboardhelper;1"]
+            .getService(Components.interfaces.nsIClipboardHelper)
+            .copyString(this.getAttribute("flashgot_media_clipboard_text"));
+        }, true);
+      }
+      var gmenu_parsed = group.menu_parsed;
       if (parsed_items.length) {
         parsed_items.sort(mo_cmp);
-        parsed_items.forEach(function(mi){delete mi["flashgot::sort_key"]; menu_parsed.appendChild(mi);});
-        menu_parsed.parentNode.setAttribute("hidden", "false");
+        parsed_items.forEach(function(mi){delete mi["flashgot::sort_key"]; gmenu_parsed.appendChild(mi);});
+        gmenu_parsed.hidden = false;
       }
       if (parsed_dash_items.length) {
+        if (do_group) {
+          // DASH streams menu - child of the non-DASH streams menu.
+          var gmenu = document.createElement("menu");
+          gmenu_parsed.insertBefore(gmenu, gmenu_parsed.firstChild);
+          gmenu.setAttribute("label", dash_menu_label);
+          gmenu.appendChild(group.menu_parsed_dash = document.createElement("menupopup"));
+        }
+        var gmenu_parsed_dash = group.menu_parsed_dash;
         parsed_dash_items.sort(mo_cmp);
         var show_av_separator = gFlashGotService.getPref("media.YouTube.dash.show_av_separator", true);
         var prev_type = parsed_dash_items[0]["flashgot::yt_dash"];
         parsed_dash_items.forEach(function(mi){
           if (show_av_separator && prev_type !== mi["flashgot::yt_dash"]) {
             show_av_separator = false;
-            menu_parsed_dash.appendChild(document.createElement("menuseparator"));
+            gmenu_parsed_dash.appendChild(document.createElement("menuseparator"));
           }
           delete mi["flashgot::sort_key"];
           delete mi["flashgot::yt_dash"];
-          menu_parsed_dash.appendChild(mi);
+          gmenu_parsed_dash.appendChild(mi);
         });
-        menu_parsed_dash.parentNode.setAttribute("hidden", "false");
+        gmenu_parsed_dash.parentNode.hidden = false;
       }
-      this._cb_mediaMenuItem_key_down = false;
-      // https://developer.mozilla.org/en-US/docs/XUL/PopupGuide/PopupKeys:
-      // A key listener added to a <menupopup> will not receive any key events.
-      // Instead, you must add a capturing key listener to the document or window
-      // if you want to listen for keys pressed within a menu.
-      window.addEventListener("keydown", this._cb_mediaMenuItem_keydown, true);
-      window.addEventListener("keyup", this._cb_mediaMenuItem_keyup, true);
-    }
+    });
+
+    this._cb_mediaMenuItem_key_down = false;
+    // https://developer.mozilla.org/en-US/docs/XUL/PopupGuide/PopupKeys:
+    // A key listener added to a <menupopup> will not receive any key events.
+    // Instead, you must add a capturing key listener to the document or window
+    // if you want to listen for keys pressed within a menu.
+    window.addEventListener("keydown", this._cb_mediaMenuItem_keydown, true);
+    window.addEventListener("keyup", this._cb_mediaMenuItem_keyup, true);
+
     return true;
   },
-  unprepareMediaMenu: function() {
+  unprepareMediaMenu: function(menu, evt) {
+    if (evt.target !== menu) { return; }
     this._cb_mediaMenuItem_key_down = false;
     window.removeEventListener("keydown", this._cb_mediaMenuItem_keydown, true);
     window.removeEventListener("keyup", this._cb_mediaMenuItem_keyup, true);
   },
+  clearMediaMenu: function() {
+    this._clearMenu(document.getElementById("flashgot-media-parsed-menu").nextSibling);
+    this._clearMenu(document.getElementById("flashgot-media-parsed-refresh_signature_func").nextSibling);
+    this._clearMenu(document.getElementById("flashgot-media-parsed-dash-popup").firstChild);
+  },
   _clearMenu: function(from) {
     if (!from) { return; }
     var pn = from.parentNode;
