@@ -16,7 +16,7 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-                             
+
 ***** END LICENSE BLOCK *****/
 
 const CI = Components.interfaces;
@@ -28,21 +28,25 @@ const NS_BINDING_ABORTED = 0x804b0002;
 const EXTENSION_ID = "{19503e42-ca3c-4c27-b1e2-9cdb2170ee34}";
 const EXTENSION_NAME = "FlashGot";
 const CHROME_NAME = "flashgot";
-const VERSION = "1.5.6.7";
+const VERSION = "1.5.6.13";
 const SERVICE_NAME = EXTENSION_NAME + " Service";
 const SERVICE_CTRID = "@maone.net/flashgot-service;1";
 const SERVICE_ID = "{2a55fc5c-7b31-4ee1-ab15-5ee2eb428cbe}";
-    
+
 // interfaces implemented by this component
-const SERVICE_IIDS = 
-[ 
+const SERVICE_IIDS =
+[
   CI.nsISupports,
   CI.nsISupportsWeakReference,
   CI.nsIObserver,
   CI.nsIURIContentListener,
   CI.nsIContentPolicy,
 ];
-
+["nsIMessageListener"].forEach(function(iface) {
+  if (iface in CI) {
+    SERVICE_IIDS.push(CI[iface])
+  }
+});
 // categories which this component is registered in
 const SERVICE_CATS = ["app-startup"];
 
@@ -72,7 +76,7 @@ const IO = {
     } catch(ex) {
       unicodeConverter.charset = "UTF-8";
     }
-    
+
     content = unicodeConverter.ConvertFromUnicode(content);
     const os = CC["@mozilla.org/network/file-output-stream;1"]
       .createInstance(CI.nsIFileOutputStream);
@@ -102,9 +106,9 @@ const INCLUDE = function(name) {
 function LAZY_INCLUDE(name) {
   if (arguments.length > 1)
     for (var j = 0, len = arguments.length; j < len; j++)
-      arguments.callee(arguments[j]);
+      LAZY_INCLUDE(arguments[j]);
   else {
-    __defineGetter__(name, function() {
+    this.__defineGetter__(name, function() {
       delete this[name];
       INCLUDE(name);
       return this[name];
@@ -125,14 +129,14 @@ var fg, singleton; // singleton
 
 const SERVICE_CREATE = function() {
   fg.wrappedJSObject = singleton = fg;
-  
+
   if ("nsIChromeRegistrySea" in CI) INCLUDE("SMUninstaller");
   fg.register();
   return fg;
 }
 
 fg = {
-  OP_ONE: 0, 
+  OP_ONE: 0,
   OP_SEL: 1,
   OP_ALL: 2,
   OP_QET: 3
@@ -142,6 +146,26 @@ fg = {
   get dom() {
     return DOM;
   },
+  get messageManager() {
+    try {
+      return CC["@mozilla.org/parentprocessmessagemanager;1"].getService(CI.nsIMessageListenerManager);
+    } catch (e) {
+      return null;
+    }
+  },
+  receiveMessage: function(m) {
+    switch(m.name) {
+      case "FlashGot::download":
+        try {
+          let links = m.data.links;
+          links.document = m.objects.document;
+          this.download(links);
+        } catch (e) {
+          this.log(e);
+        }
+      break;
+    }
+  },
   register: function() {
     const os = this.observerService;
     os.addObserver(this, SHUTDOWN, false);
@@ -150,6 +174,9 @@ fg = {
     os.addObserver(this, "private-browsing", false);
     os.addObserver(this, "browser:purge-session-history", false);
     os.addObserver(this, "last-pb-context-exited", false);
+
+    var mm = this.messageManager;
+    if (mm) mm.addWeakMessageListener("FlashGot::download", this);
   },
   unregister: function() {
     try {
@@ -161,6 +188,8 @@ fg = {
       os.removeObserver(this, "private-browsing");
       os.removeObserver(this, "browser:purge-session-history");
       os.removeObserver(this, "last-pb-context-exited");
+      var mm = this.messageManager;
+      if (mm) mm.removeWeakMessageListener("FlashGot::download", this);
     } catch(ex) {
       this.log("Error unregistering service as observer: " + ex);
     }
@@ -181,7 +210,7 @@ fg = {
         case "xpcom-shutdown":
           this.unregister();
           break;
-        case SHUTDOWN: 
+        case SHUTDOWN:
           this.cleanup();
           break;
         case STARTUP:
@@ -216,7 +245,7 @@ fg = {
       }
     }
   },
-  
+
   privatize: function(channel) {
     if (("nsIPrivateBrowsingChannel" in CI) && channel instanceof CI.nsIPrivateBrowsingChannel) {
       channel.setPrivate(true);
@@ -224,15 +253,15 @@ fg = {
       channel.loadFlags |= channel.INHIBIT_PERSISTENT_CACHING;
     }
   },
-  
+
   isPrivate: function(window) {
     var privacyContext = window && window.QueryInterface(CI.nsIInterfaceRequestor)
       .getInterface(CI.nsIWebNavigation);
-    return ((privacyContext instanceof CI.nsILoadContext) && "usePrivateBrowsing" in privacyContext) 
+    return ((privacyContext instanceof CI.nsILoadContext) && "usePrivateBrowsing" in privacyContext)
             ? privacyContext.usePrivateBrowsing
             : this.inPrivate;
   },
-  
+
   inPrivate: false,
   uninstalling: false
 ,
@@ -249,12 +278,12 @@ fg = {
              }
           }
         break;
-        
+
         case "autoStart":
         case "interceptAll":
           this.interceptor[name] = this.getPref(name);
         break;
-      
+
       }
     }
   }
@@ -269,7 +298,7 @@ fg = {
     }
   }
 ,
-  
+
   get defaultDM() {
     return this.getUPref("defaultDM", null);
   },
@@ -280,15 +309,15 @@ fg = {
   get isWindows() {
     return this._lazyPlatformCheck("isWin", "WinD");
   },
-  
+
   get isMac() {
     return this._lazyPlatformCheck("isMac", "UsrDsk");
   },
-  
+
   get isLinux() {
     return this._lazyPlatformCheck("isLinux", "XDGDesk");
   },
-  
+
   _lazyPlatformCheck: function(prop, folder) {
     delete this[prop];
     var is = false;
@@ -297,7 +326,7 @@ fg = {
     } catch(e) {}
     return this[prop] = is;
   },
-  
+
   get directoryService() {
     delete this.directoryService;
     return this.directoryService = CC["@mozilla.org/file/directory_service;1"]
@@ -309,22 +338,22 @@ fg = {
       CC["@mozilla.org/cookiemanager;1"].getService(CI.nsICookieManager2);
   }
 ,
-  
+
   get java() {
     var java;
-    
+
     try {
       java = this.prefs.getComplexValue("java", CI.nsILocalFile);
     } catch(e) {
-    
+
       if (this.isWindows) {
         java = this.directoryService.get("WinD", CI.nsIFile);
         java.append("System32");
         java.append("javaw.exe");
       } else {
-      
+
         java = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
-        
+
         if (this.isMac) {
           java.initWithPath("/usr/bin/java");
         } else {
@@ -338,7 +367,7 @@ fg = {
         }
       }
     }
-    
+
     delete this.java;
     return this.java = java;
   }
@@ -376,7 +405,7 @@ fg = {
         document.getElementById(id).style.display = "none";
       } catch(e) {}
     }
-    (document._FlashGot_NativeUI_styleSheets || 
+    (document._FlashGot_NativeUI_styleSheets ||
       (document._FlashGot_NativeUI_styleSheets = [])
     ).push(s);
   },
@@ -396,11 +425,11 @@ fg = {
      }
      document._FlashGot_NativeUI_styleSheets = null;
   },
-  
+
   cursor: function(b) {
     DOM.updateStyleSheet('@-moz-document regexp("https?:.*") { body * { cursor: url(chrome://flashgot/skin/icon32.png), auto !important }}', b);
   },
-  
+
   _httpServer: null,
   get httpServer() {
     if (typeof(FlashGotHttpServer) != "function") {
@@ -415,9 +444,9 @@ fg = {
     return s.replace(/[\u0000-\u001f\/\\":<>|?*]/g, '_');
   },
   createDownloadFile: function(parentFile, fileSpec, overwrite) {
-    
+
     if (fileSpec instanceof CI.nsIURL) {
-     
+
       fileSpec = fileSpec.fileName || fileSpec.filePath.replace(/.*?([^\/]*)\/?$/, '$1') || fileSpec.query || fileSpec.host;
       try {
         fileSpec = decodeURIComponent(fileSpec);
@@ -427,14 +456,14 @@ fg = {
     } else if (fileSpec instanceof CI.nsIURI) {
       return null;
     }
-    
+
     if (!fileSpec) return null;
-    
+
     fileSpec = this.sanitizeFileName(fileSpec);
-    
+
     var file = parentFile.clone();
     file.append(fileSpec);
-            
+
     if (!overwrite || file.exists() && file.isDirectory()) {
       for (;;) {
         if(!file.exists()) {
@@ -447,7 +476,7 @@ fg = {
           break;
         } else { // rename
           var m = file.leafName.match(/(.*?)(?:\((\d+)\))?(\.[^\.]+$|$)/);
-          file.leafName = m[1] + "(" + ((m[2] && parseInt(m[2]) || 0) + 1) + ")" + m[3]; 
+          file.leafName = m[1] + "(" + ((m[2] && parseInt(m[2]) || 0) + 1) + ")" + m[3];
         }
       }
     }
@@ -461,76 +490,76 @@ fg = {
   }
 ,
   download: function(links, opType, dmName) {
-    
+
     switch (links.length) {
-      case 0: 
+      case 0:
         return false;
-      case 1: 
-        opType = this.OP_ONE; 
+      case 1:
+        opType = this.OP_ONE;
         break;
       default:
         if (!opType) opType = this.OP_SEL;
     }
-    
-    
+
+
     var win = links.document && links.document.defaultView ||
       DOM.mostRecentBrowserWindow;
     if (win && win.content) win = win.content;
-    
+
     var privacyContext = win && win.QueryInterface(CI.nsIInterfaceRequestor)
       .getInterface(CI.nsIWebNavigation);
     if ((privacyContext instanceof CI.nsILoadContext) && "usePrivateBrowsing" in privacyContext) {
       this.inPrivate = privacyContext.usePrivateBrowsing;
     }
     links.privacyContext = privacyContext || null;
-    
+
     if (!dmName) dmName = this.defaultDM;
     const dm = this.DMS[dmName] || this.DMS[this.getString("dm.builtIn")];
     if (!dm) {
       this.log("FlashGot error: no download manager selected!");
       return false;
     }
-    
+
     // surrogate missing attributes
-    
+
     if (!links.progress) {
       links.progress = { update: function() {} };
     } else {
       links.progress.update(12);
     }
-    
-  
-    this.delayExec(function(t) { fg._downloadDelayed(links, opType, dm); }); 
+
+
+    this.delayExec(function(t) { fg._downloadDelayed(links, opType, dm); });
     return true;
   },
-  
+
   _downloadDelayed: function(links, opType, dm) {
 
     const encodedURLs = this.getPref(dm.getPref("encode"), this.getPref("encode", true));
 
     const extFilter = this.getPref("extfilter", false) && !this.interceptor.interceptAll ?
         new RegExp("\.(" +
-          this.extensions.join("|").replace(/[^\w-|]/,"") + 
+          this.extensions.join("|").replace(/[^\w-|]/,"") +
           ")\\b", "i") : null;
-    
+
     var logMsg = "Processing " + links.length + " links ";
     if (this.logEnabled && typeof(links.startTime) == "number") {
       logMsg += "scanned in ms" + (Date.now() - links.startTime);
     }
-    
-    
+
+
 
     if (!links.startTime) links.startTime = Date.now();
     const pg = links.progress;
-    
+
     var len = links.length;
-    
+
     var filters = null;
-    
+
     if (len > 1) {
       filters = [];
-      
-      const isValid = dm.isValidLink; 
+
+      const isValid = dm.isValidLink;
       if (isValid)  filters.push(function(href) { return isValid(href) });
       if (extFilter) filters.push(function(href) { return extFilter.test(href) });
 
@@ -543,14 +572,14 @@ fg = {
         filters = null;
       }
     }
-    
-    
+
+
     const map = {};
     pg.update(10);
-    
+
     const stripHash = dm.getPref("stripHash", false);
     var cs = links.document && links.document.characterSet || null;
-    
+
     var j, l, href, ol, pos1, pos2;
     var postData;
     for (j = 0; j < len; j++) {
@@ -562,7 +591,7 @@ fg = {
       }
 
       href = l.href;
-      
+
       if (/^javascript:/i.test(href)) {
         try {
           l.href = href = href.match(/(['"])(https?:\/\/.*?)\1/)[2];
@@ -571,13 +600,13 @@ fg = {
         }
         if (!href) continue;
       }
-      
+
       if (filters && !filters.doFilter(href)) continue;
-      
+
       if (l.description) l.description = l.description.replace(/\s+/g, ' ');
       if (l.fname) l.fname = this.sanitizeFileName(l.fname);
       l._pos = j;
-      
+
       ol = map[href];
       if (ol) { // duplicate, keep the longest description
         if (ol.description.length < l.description.length) {
@@ -586,26 +615,26 @@ fg = {
         }
       } else {
         map[href] = l;
-        
+
         // encoding checks
         try {
-          if (encodedURLs) { 
+          if (encodedURLs) {
             href = IOS.newURI(href, cs, null).asciiSpec; // punycode + % encoding
             // workaround for malformed hash urls
-           
+
             while ((pos1 = href.indexOf("#")) > -1 // has fragment?
               && href[pos1 + 1] != "!" // skip metalinks!
-              && (href.indexOf("?") > pos1 || pos1 != href.lastIndexOf('#')) // fragment before query or double fragment ? 
+              && (href.indexOf("?") > pos1 || pos1 != href.lastIndexOf('#')) // fragment before query or double fragment ?
             ) {
               href = href.substring(0, pos1) + '%23' + href.substring(pos1 + 1);
             }
-            
+
             l.href = href;
-          } else {  
+          } else {
             l.href = decodeURI(href);
           }
           if (stripHash) l.href = l.href.replace(/#.*/g, '');
-          
+
         } catch(e) {
           dump("Problem "
             + ( encodedURLs ? "escaping" : "unescaping")
@@ -614,24 +643,24 @@ fg = {
       }
     }
     pg.update(25);
-    
+
     links.length = 0;
     for (href in map) links[links.length] = map[href];
-    
+
     if (this.getPref("noDesc", false) || dm.getPref("noDesc", false)) {
       for (j = links.length; j-- > 0;) links[j].description = '';
     } else if(dm.asciiFilter) {
       for (j = links.length; j-- > 0;) {
         l = links[j];
-        if(l.description) 
+        if(l.description)
           l.description = l.description.replace(/[^\u0020-\u007f]/g, "") || l.href;
       }
     }
-    
-    
+
+
     this._processRedirects(links, opType, dm);
   },
-  
+
   get RedirectContext() {
     delete this.RedirectContext;
     INCLUDE('RedirectContext');
@@ -639,28 +668,28 @@ fg = {
   },
   _processRedirects: function(links, opType, dm) {
     links.progress.update(30);
-    this.delayExec(function() {  
+    this.delayExec(function() {
       new fg.RedirectContext(links, opType, dm, function(processedBy) {
         links.redirProcessedBy = processedBy;
         fg._sendToDM(links, opType, dm);
       }).process();
     });
   },
-  
+
   _sendToDM: function(links, opType, dm) {
-    
+
     if (this.getPref("httpauth", false)) {
       dm.log("Adding authentication info");
       this._addAuthInfo(links);
     }
-    
+
     if (dm.metalinkSupport && this.getPref("metalink", true)) {
       dm.log("Adding metalink info");
       if (this._processMetalinks(links)) {
         opType = this.OP_SEL; // force "ask path"
       }
     }
-    
+
     if (links.length > 1) {
       dm.log("Sorting again "+links.length+" links");
       links.sort(function(a,b) {
@@ -669,29 +698,29 @@ fg = {
       });
       opType = this.OP_SEL;
     }
-    
+
     this._addQsSuffix(links);
-    
+
     links.progress.update(70);
-    
+
     dm.log("Preprocessing done in ms" + (Date.now() - links.startTime) );
-    
+
     // "true" download
     this.delayExec(function() {
         dm.log("Starting dispatch");
         var startTime = Date.now();
-    
+
         dm.download(links, opType);
 
         var now = Date.now();
         var logMsg = "Dispatch done in ms" + (now - startTime);
-        if (typeof(links.startTime) == "number") { 
+        if (typeof(links.startTime) == "number") {
           logMsg += "\nTotal processing time: ms" + (now - links.startTime);
-        }  
+        }
         dm.log(logMsg);
       });
   },
-  
+
   _addQsSuffix: function(links) {
     var suffix = this.getPref("queryStringSuffix");
     if (suffix) {
@@ -705,7 +734,7 @@ fg = {
       }
     }
   },
-  
+
   _addAuthInfo: function(links) {
     const httpAuthManager = CC['@mozilla.org/network/http-auth-manager;1']
                               .getService(CI.nsIHttpAuthManager);
@@ -723,7 +752,7 @@ fg = {
       try {
         httpAuthManager.getAuthIdentity(uri.scheme, uri.host, uri.port, null, null, uri.path, udom, uname, upwd);
       } catch(e) {
-        
+
         if (uri.scheme !== "ftp" || !uri.username)
           continue;
 
@@ -731,18 +760,18 @@ fg = {
           .getNewAuthPrompter(null).QueryInterface(CI.nsIAuthPrompt);
         var port = (uri.port < 0 || uri.port == 21 ? "" : ":" + uri.port);
         var realm = uri.scheme + "://" + uri.username + "@" + uri.host + port;
-        upwd.value = (realm in seen) ? seen[realm] : ''; 
-        if (!(upwd.value || 
+        upwd.value = (realm in seen) ? seen[realm] : '';
+        if (!(upwd.value ||
             prompter.promptPassword("Password", realm + " password", realm, prompter.SAVE_PASSWORD_NEVER, upwd))
           )
           continue;
-        
+
         uname.value = uri.username;
         seen[realm] = upwd.value;
       }
       l.userPass = encodeURIComponent(uname.value) + ":" + encodeURIComponent(upwd.value);
       fg.log("Authentication data " + l.userPass + " for " + l.href + " added.");
-      l.href = uri.scheme + "://" + l.userPass + "@" + 
+      l.href = uri.scheme + "://" + l.userPass + "@" +
                uri.host + (uri.port < 0 ? "" : (":" + uri.port)) + uri.spec.substring(uri.prePath.length);
     }
   },
@@ -756,7 +785,7 @@ fg = {
        if (pos < 0) continue;
        parts = href.substring(pos + 2).split("#!");
        if (parts[0].indexOf("metalink3!") == 0) continue; // per Ant request
-       
+
        hasMetalinks = true;
        l.metalinks = [];
        for (k = 0; k < parts.length; k++) {
@@ -782,14 +811,14 @@ fg = {
   _timers: [],
   delayExec: function(callback, delay) {
     var timer = CC["@mozilla.org/timer;1"].createInstance(CI.nsITimer);
-    timer.initWithCallback({ 
+    timer.initWithCallback({
         notify: this.delayedRunner,
         context: { callback: callback, args: Array.prototype.slice.call(arguments, 2), self: this }
       }, delay || 1, 0);
     this._timers.push(timer);
   },
-  
-  
+
+
   delayedRunner: function(timer) {
     var ctx = this.context;
     try {
@@ -827,7 +856,7 @@ fg = {
     }
   }
 ,
-  
+
   get prefService() {
     delete this.prefService;
     return this.prefService = CC["@mozilla.org/preferences-service;1"]
@@ -956,12 +985,12 @@ fg = {
       }
     };
   },
-  
+
   logFlush: function() {
     this.logStream._handOver();
     this.logFlush();
   },
-  
+
   logStream: { // we buffer in ram until we reach 16KB or we get flushed, in order to reduce startup footprint
     _buf: "",
     writeString: function(msg) {
@@ -980,7 +1009,7 @@ fg = {
       fg.logStream.writeString(this._buf);
     }
   },
-  
+
   log: function(msg) {
     if (this.logEnabled) {
       try {
@@ -1015,7 +1044,7 @@ fg = {
       if (this.logFile) this.logFile.remove(true);
       this.logInit();
     } catch(ex) { dump(ex.message); }
-  } 
+  }
 ,
   get windowMediator() {
     return CC["@mozilla.org/appshell/window-mediator;1"
@@ -1028,7 +1057,7 @@ fg = {
 ,
   getBrowserWindow: function(document) {
     var w = document && document.defaultView && DOM.getChromeWindow(document.defaultView.top) || DOM.mostRecentBrowserWindow;
-    return w.wrappedJSObject || w; 
+    return w.wrappedJSObject || w;
   }
 ,
   get prefs() {
@@ -1042,50 +1071,50 @@ fg = {
     this.hasDMS = true;
     return this.DMS = this.checkDownloadManagers(true, false);
   },
-  
+
   get tmpDir() {
-    
+
     function prepareTmp(t, create) {
       if (!(create || t.exists())) throw "Temporary dir " + t.path + " doesn't exist.";
-      
+
       t.append("flashgot." + encodeURI(fg.profDir.leafName).replace(/%/g,"_"));
       if (t.exists()) {
        if (!(t.isDirectory() && t.isWritable())) t.createUnique(1, B8('700'));
       } else {
         t.create(1, B8('700'));
-      } 
+      }
       return t;
     }
-    
+
     delete this.tmpDir;
     try {
       return this.tmpDir = prepareTmp(this.prefs.getComplexValue("tmpDir", CI.nsILocalFile), false);
     } catch (e) {}
-    
+
     return this.tmpDir = prepareTmp(this.directoryService.get("TmpD", CI.nsILocalFile), true);
-   
+
   },
   get profDir() {
     delete this.profDir;
     return this.profDir = this.directoryService.get("ProfD", CI.nsIFile);
   },
-  
+
   _initialized: false,
   init: function() {
     if (this._initialized) return;
 
     if (this.smUninstaller) this.smUninstaller.check();
-  
+
     const os = this.observerService;
-    
+
     os.addObserver(this, "em-action-requested", false);
-    
+
     try {
       const startTime = Date.now();
-       
+
       this.prefs.addObserver("", this, false);
       this.syncPrefs();
-      
+
       this.log("Per-session init started");
 
       this.interceptor = new HttpInterceptor();
@@ -1097,13 +1126,13 @@ fg = {
           .addProgressListener(MediaSniffer, CI.nsIWebProgress.NOTIFY_STATE_NETWORK | CI.nsIWebProgress.NOTIFY_LOCATION);
       }
       this.interceptor.setup();
-    
+
       this.log("Per-session init done in " + (Date.now() - startTime) + "ms");
     } catch(initEx) {
       this._initException = initEx;
       try { this.log(initEx); } catch(e) {}
     }
-    this._initialized = true; 
+    this._initialized = true;
   },
 
   dispose: function() {
@@ -1118,7 +1147,7 @@ fg = {
       CC['@mozilla.org/docloaderservice;1'].getService(CI.nsIWebProgress)
         .removeProgressListener(MediaSniffer);
     } catch(e) {}
-    
+
     this._initialized = false;
   }
 ,
@@ -1146,30 +1175,30 @@ fg = {
   }
 ,
   sortDMS: function() {
-    FlashGotDM.dms.sort(function(a,b) { 
-      a = a.priority || a.name.toLowerCase(); 
+    FlashGotDM.dms.sort(function(a,b) {
+      a = a.priority || a.name.toLowerCase();
       b = b.priority || b.name.toLowerCase();
-      return a > b ? 1 : a < b ?-1 : 0; 
+      return a > b ? 1 : a < b ?-1 : 0;
     });
   }
-, 
+,
   checkDownloadManagers: function(init, detect) {
     INCLUDE("DMS");
-    
+
     var t = Date.now();
-    
+
     if (init || detect) FlashGotDM.init(this);
-    
+
     const dms = FlashGotDM.dms;
     dms.found = false;
     var defaultDM = this.defaultDM;
     if (!dms[defaultDM]) defaultDM = null;
-    
+
     detect = detect || this.getPref("detect.auto", true);
- 
+
     var j, dm;
     var cache = this.getUPref('detect.cache', '').split(',');
-    
+
     var cacheLen = cache.length;
     if (!detect) {
       for (j = dms.length; j-- > 0;) {
@@ -1184,7 +1213,7 @@ fg = {
         }
       }
     }
-    
+
     cache = [];
 
     var firstSupported = null;
@@ -1201,13 +1230,13 @@ fg = {
         }
       }
     }
-    
+
     this.setUPref("detect.cache", cache.map(function(dm) { return dm.name; }).join(","));
-    
+
     if (cacheLen > 0 && cache.length > cacheLen && defaultDM && !dms[defaultDM].autoselect) { // new DM added
       defaultDM = null;
     }
-    
+
     if (!defaultDM && firstSupported) {
       while (!firstSupported.autoselect && cache.length) {
         firstSupported = cache.pop();
@@ -1217,9 +1246,9 @@ fg = {
     } else if(!dms.found) {
       this.log("Serious warning! no supported download manager found...");
     }
-    
+
     this.log("Download managers detection done in " + (Date.now() - t) + "ms");
-    
+
     return dms;
   }
 ,
@@ -1239,19 +1268,19 @@ fg = {
       dump("Error shredding temp files: " + e + "\n");
     }
   },
-  
+
   doomPrivateFile: function(f) {
     if (this.inPrivate || this.getPref("shredding"))
       this.delayExec(function() { fg.shredFiles(f); }, 5000);
   },
-  
+
   shredFiles: function(files) {
     INCLUDE("Shredder");
     if (!("push" in files)) files = [files];
     Shredder.shred(files);
   },
-  
-  
+
+
   cleanup: function() {
     if (this._cleaningup) return;
     try {
@@ -1260,18 +1289,18 @@ fg = {
       if (this._httpServer) {
         this._httpServer.shutdown();
       }
-      
+
       try {
         if (typeof FlashGotDM === "object") FlashGotDM.cleanup(this.uninstalling);
       } catch(eexx) {
         dump(eexx.message);
       }
-      
+
       if (this.tmpDir && this.tmpDir.exists()) {
         try {
           if ((this.inPrivate || this.getPref("shredding")))
           {
-            this._shredTempFiles(); 
+            this._shredTempFiles();
           } else {
             this.tmpDir.remove(true);
           }
@@ -1309,7 +1338,7 @@ fg = {
           var logStart=-1;
           while ((buffer = sis.read(5000))) {
             content += buffer;
-            if ((logStart=content.indexOf("\n*** Log start at ")) > -1) { 
+            if ((logStart=content.indexOf("\n*** Log start at ")) > -1) {
               content=content.substring(logStart);
               break;
             }
@@ -1322,7 +1351,7 @@ fg = {
             os.write(content,content.length);
             while ((buffer = sis.read(20000))) {
               os.write(buffer,buffer.length);
-            } 
+            }
             os.close();
           }
           sis.close();
@@ -1337,7 +1366,7 @@ fg = {
     this._cleaningup = false;
     this.dispose();
   }
-,  
+,
   logHex: function(s) {
     var cc = [];
     for(var j = 0, len = s.length; j < len; j++) {
@@ -1347,9 +1376,9 @@ fg = {
   }
 ,
   showDMSReference: function() {
-    this.getWindow().open("http://flashgot.net/dms","_blank");
+    this.getWindow().open("https://flashgot.net/dms","_blank");
   },
-  
+
   getMedia: function(w) {
     return "_flashgotMedia" in w ? w._flashgotMedia : null;
   }
@@ -1366,9 +1395,9 @@ fg = {
   versionChecked: false,
   checkVersion: function() {
     if (this.versionChecked) return;
-    
+
     this.versionChecked = true;
-    
+
     const ver =  this.VERSION;
     const prevVer = this.getPref("version", "");
     if ((this.firstRun = prevVer !== ver)) {
@@ -1379,36 +1408,36 @@ fg = {
       this.setPref("version", ver);
       this.savePrefs();
     }
-    
+
     const betaRx = /(?:a|alpha|b|beta|pre|rc)\d*$/; // see http://viewvc.svn.mozilla.org/vc/addons/trunk/site/app/config/constants.php?view=markup#l431
     if (prevVer.replace(betaRx, "") != ver.replace(betaRx, "")) {
 
       if (this.getPref("firstRunRedirection", true)) {
         const name = EXTENSION_NAME;
         const domain = name.toLowerCase() + ".net";
-        IOS.newChannel("http://" + domain + "/-", null, null).asyncOpen({ // DNS prefetch
+        IOS.newChannel("https://" + domain + "/-", null, null).asyncOpen({ // DNS prefetch
           onStartRequest: function() {},
           onStopRequest: function() {
             var browser = DOM.mostRecentBrowserWindow.getBrowser();
             if (typeof(browser.addTab) != "function") return;
-           
-            
-            var url = "http://" + domain + "/?ver=" + ver;
+
+
+            var url = "https://" + domain + "/?ver=" + ver;
             var hh = "X-IA-Post-Install: " + name + " " + ver;
             if (prevVer) {
               url += "&prev=" + prevVer;
               hh += "; updatedFrom=" + prevVer;
             }
             hh += "\r\n";
-            
+
             var hs = CC["@mozilla.org/io/string-input-stream;1"] .createInstance(CI.nsIStringInputStream);
-            hs.setData(hh, hh.length); 
-            
-            
+            hs.setData(hh, hh.length);
+
+
             var b = (browser.selectedTab = browser.addTab()).linkedBrowser;
             b.stop();
             b.webNavigation.loadURI(url, CI.nsIWebNavigation.LOAD_FLAGS_NONE, null, null, hs);
-            
+
           },
           onDataAvailable: function() {}
         }, {});
@@ -1419,5 +1448,5 @@ fg = {
 
 fg.__global__ = this;
 fg._e = function(f) {
-  return eval("(" + f + ")()"); 
-}
\ No newline at end of file
+  return eval("(" + f + ")()");
+}
